@@ -9,7 +9,7 @@ mkdir($Path) unless -e $Path;
 sub SessionInit() {
     my %Session=();
     $SID=GetCookie('sid'); $SID ~~ s:perl5:g/[^A-Z]//;
-    if (!-e catfile($Path, $SID)) { $SID=''; }
+    if !(-e catfile($Path, $SID)) { $SID=''; }
     $SID=IDGenerate() if $SID eq '';
     SessionDecode(slurp(catfile($Path, $SID)));
     CookiesAdd('sid',$SID);
