@@ -13,7 +13,7 @@ my Str $EMPTY_STR is readonly = q{};
 ###########################################################################
 ###########################################################################
 
-package Locale::KeyedText-1.72.1 {
+package Locale::KeyedText-1.73.0 {
     # Note: This given version applies to all of this file's packages.
 } # package Locale::KeyedText
 
@@ -48,6 +48,15 @@ submethod BUILD (Str :$msg_key!, Any :%msg_vars? = {}) {
 
 ###########################################################################
 
+method export_as_hash () returns Hash {
+    return {
+        'msg_key'  => $!msg_key,
+        'msg_vars' => {%!msg_vars},
+    };
+}
+
+###########################################################################
+
 method get_msg_key () returns Str {
     return $!msg_key;
 }
@@ -82,7 +91,7 @@ method as_debug_string () returns Str {
 my method _die_with_msg (Str $msg_key!, Any %msg_vars? is ref = {}) {
     %msg_vars{'CLASS'} = 'Locale::KeyedText::Message';
     die Locale::KeyedText::Message.new(
-        'msg_key' => $msg_key, 'msg_vars' => %msg_vars );
+        msg_key => $msg_key, msg_vars => %msg_vars );
 }
 
 my method _assert_arg_str (Str $meth!, Str $arg!, Str $val!) {
@@ -138,6 +147,15 @@ submethod BUILD (Str :@set_names!, Str :@member_names!) {
 
 ###########################################################################
 
+method export_as_hash () returns Hash {
+    return {
+        'set_names'    => [@!set_names],
+        'member_names' => [@!member_names],
+    };
+}
+
+###########################################################################
+
 method get_set_names () returns Array of Str {
     return [@!set_names];
 }
@@ -296,7 +314,7 @@ method interpolate_vars_into_template_text
 my method _die_with_msg (Str $msg_key!, Any %msg_vars? is ref = {}) {
     %msg_vars{'CLASS'} = 'Locale::KeyedText::Translator';
     die Locale::KeyedText::Message.new(
-        'msg_key' => $msg_key, 'msg_vars' => %msg_vars );
+        msg_key => $msg_key, msg_vars => %msg_vars );
 }
 
 my method _assert_arg_str (Str $meth!, Str $arg!, Str $val!) {
@@ -362,7 +380,7 @@ Refer to user messages in programs by keys
 
 =head1 VERSION
 
-This document describes Locale::KeyedText version 1.72.1.
+This document describes Locale::KeyedText version 1.73.0.
 
 It also describes the same-number versions of Locale::KeyedText::Message
 ("Message") and Locale::KeyedText::Translator ("Translator").
@@ -382,9 +400,9 @@ your code; instead refer to other above-named packages in this file.>
     sub main () {
         # Create a translator.
         my Locale::KeyedText::Translator $translator .= new(
-            'set_names' => ['MyLib::Lang::', 'MyApp::Lang::'],
+            set_names => ['MyLib::Lang::', 'MyApp::Lang::'],
                 # set package prefixes for localized app components
-            'member_names' => ['Eng', 'Fr', 'De', 'Esp'],
+            member_names => ['Eng', 'Fr', 'De', 'Esp'],
                 # set list of available languages in order of preference
         );
 
@@ -392,7 +410,7 @@ your code; instead refer to other above-named packages in this file.>
         # languages that has a matching template available.
         print $translator.translate_message(
             Locale::KeyedText::Message.new(
-                'msg_key' => 'MYAPP_PROMPT' ) );
+                msg_key => 'MYAPP_PROMPT' ) );
 
         # Read two numbers from the user.
         my Num ($first, $second) = =$*IN;
@@ -411,8 +429,8 @@ your code; instead refer to other above-named packages in this file.>
         # the first possible language.  For example, if the user
         # inputs '3' and '4', it the output will be '3 plus 4 equals 7'.
         print $translator.translate_message(
-            Locale::KeyedText::Message.new( 'msg_key' => 'MYLIB_RESULT',
-                'msg_vars' => { 'FIRST' => $first, 'SECOND' => $second,
+            Locale::KeyedText::Message.new( msg_key => 'MYLIB_RESULT',
+                msg_vars => { 'FIRST' => $first, 'SECOND' => $second,
                 'RESULT' => $sum } ) );
     }
 
@@ -643,10 +661,10 @@ provided (it defaults to empty if the argument is not provided).
 Some example usage:
 
     my Locale::KeyedText::Message $message .= new(
-        'msg_key' => 'FOO_GOT_NO_ARGS' );
+        msg_key => 'FOO_GOT_NO_ARGS' );
     my Locale::KeyedText::Message $message2 .= new(
-        'msg_key' => 'TABLE_COL_NO_EXIST',
-        'msg_vars' => {
+        msg_key => 'TABLE_COL_NO_EXIST',
+        msg_vars => {
             'GIVEN_TABLE_NAME' => $table_name,
             'GIVEN_COL_NAME' => $col_name,
         } );
@@ -662,6 +680,11 @@ A Message object has these methods:
 
 =over
 
+=item C<export_as_hash()>
+
+This method returns a deep copy of this Message as a Hash ref of 2
+elements, which correspond to the 2 named parameters of new().
+
 =item C<get_msg_key()>
 
 This method returns the Message Key attribute of its object.
@@ -819,10 +842,10 @@ The Set Names property of the new object is set from the named parameter
 Some example usage:
 
     my Locale::KeyedText::Translator $translator .= new(
-        'set_names' => ['Foo::L::','Bar::L::'],
-        'member_names' => ['Eng', 'Fre', 'Ger'] );
+        set_names => ['Foo::L::','Bar::L::'],
+        member_names => ['Eng', 'Fre', 'Ger'] );
     my Locale::KeyedText::Translator $translator2 .= new(
-        'set_names' => ['Foo::L::'], 'member_names' => ['Eng'] );
+        set_names => ['Foo::L::'], member_names => ['Eng'] );
 
 Note that a Translator object does not permit changes to its attributes;
 they must all be set when the object is constructed.  If you want to
@@ -835,6 +858,11 @@ A Translator object has these methods:
 
 =over
 
+=item C<export_as_hash()>
+
+This method returns a deep copy of this Translator as a Hash ref of 2
+elements, which correspond to the 2 named parameters of new().
+
 =item C<get_set_names()>
 
 This method returns all Set Names elements in this object as an array ref.
@@ -937,7 +965,7 @@ None reported.
 
 These Perl 6 packages are the initial main dependents of Locale::KeyedText:
 L<Rosetta::Model>, L<Rosetta>, L<Rosetta::Validator>,
-L<Rosetta::Engine::Native>.
+L<Rosetta::Engine::Example>, L<Rosetta::Shell>.
 
 =head1 BUGS AND LIMITATIONS
 
