@@ -8,6 +8,7 @@ import UserRuleXml
 import UserRulesGeneric
 import UserRuleGhcBinary
 import qualified RulePerl5
+import qualified RuleYAML
 import qualified RuleJSON
 import qualified RuleUtility 
 import qualified RuleFunctorM
@@ -16,5 +17,5 @@ import qualified RuleMonoid
 
 -- add your rules to this list
 userRules :: [RuleDef]
-userRules = userRulesXml ++ userRulesBinary ++ userRulesGeneric ++ userRulesGhcBinary  ++ RuleUtility.rules ++ RuleFunctorM.rules ++ RuleMonoid.rules ++ RulePerl5.rules ++ RuleJSON.rules
+userRules = userRulesXml ++ userRulesBinary ++ userRulesGeneric ++ userRulesGhcBinary  ++ RuleUtility.rules ++ RuleFunctorM.rules ++ RuleMonoid.rules ++ RulePerl5.rules ++ RuleJSON.rules ++ RuleYAML.rules
 
