@@ -1,9 +1,9 @@
 {-# OPTIONS_GHC -fglasgow-exts -fallow-undecidable-instances -fno-warn-orphans -funbox-strict-fields -cpp #-}
-{-# OPTIONS_GHC -#include "../../UnicodeC.h" #-}
 
 module Pugs.CodeGen.PIL1 (genPIL1) where
 import Pugs.Internals
 import Pugs.AST
+import Pugs.PIL1.Instances ()
 import Pugs.PIL1
 import Pugs.Compile
 
