@@ -1,5 +1,4 @@
-{-# OPTIONS_GHC -fglasgow-exts -fallow-undecidable-instances -fno-warn-orphans -funbox-strict-fields -cpp #-}
-{-# OPTIONS_GHC -#include "../UnicodeC.h" #-}
+{-# OPTIONS_GHC -fglasgow-exts -fallow-undecidable-instances -fno-warn-orphans -funbox-strict-fields -cpp -fno-warn-deprecations #-}
 
 {-|
     Compiler interface.
@@ -22,7 +21,6 @@ module Pugs.Compile (
 import Pugs.AST
 import Pugs.Internals
 import Pugs.Types
-import Pugs.Eval
 import Pugs.Monads
 import Pugs.PIL1
 import Emit.PIR
@@ -142,9 +140,12 @@ instance Compile (String, [(TVar Bool, TVar VRef)]) PIL_Expr where
     compile (name, _) = return $ PRawName name
 
 instance Compile Exp PIL_Stmts where
-    compile (Ann (Pos _) rest) = compile rest -- fmap (PPos pos rest) $ compile rest
-    compile (Ann (Cxt cxt) rest) = enter cxt $ compile rest
     -- XXX: pragmas?
+    compile (Ann Pos{} rest) = compile rest -- fmap (PPos pos rest) $ compile rest
+    compile (Ann Prag{} rest) = compile rest -- fmap (PPos pos rest) $ compile rest
+    compile (Ann (Cxt cxt) rest) = enter cxt $ compile rest
+    compile (Ann _ rest) = compile rest
+    compile (Sym _ "" rest) = compile rest
     compile (Stmts (Pad SOur _ exp) rest) = do
         compile $ mergeStmts exp rest
     compile (Stmts (Pad scope pad exp) rest) = do
@@ -203,6 +204,9 @@ instance Compile Exp PIL_Stmt where
     compile (Ann (Pos pos) rest) = fmap (PPos pos rest) $ compile rest
     compile (Ann (Cxt cxt) rest) = enter cxt $ compile rest
     -- XXX: pragmas?
+    compile (Ann Prag{} rest) = compile rest -- fmap (PPos pos rest) $ compile rest
+    compile (Ann _ rest) = compile rest
+    compile (Sym _ "" rest) = compile rest
     compile Noop = return PNoop
     compile (Val val) = do
         cxt     <- asks envContext
@@ -217,8 +221,8 @@ instance Compile Exp PIL_Stmt where
         compile (Syn "loop" $ [emptyExp, Val (VBool True), emptyExp, exp])
     compile (Syn "loop" [pre, cond, post, (Syn "block" [body])]) = do
         preC    <- compile pre
-        -- loop ...; ; ... {...} ->
-        -- loop ...; bool::true; ... {...}
+        -- loop (...; ; ...) {...} ->
+        -- loop (...; True; ...) {...}
         let cond' | unwrap cond == Noop
                   = return $ PStmts (PStmt . PLit . PVal $ VBool True) PNil
                   | otherwise
@@ -275,8 +279,11 @@ instance (Compile a b, Compile a c, Compile a d) => Compile [a] (b, c, d) where
     compile x = compError x
 
 instance Compile Exp PIL_LValue where
-    compile (Ann (Pos _) rest) = compile rest -- fmap (PPos pos rest) $ compile rest
+    compile (Ann Pos{} rest) = compile rest -- fmap (PPos pos rest) $ compile rest
+    compile (Ann Prag{} rest) = compile rest
     compile (Ann (Cxt cxt) rest) = enter cxt $ compile rest
+    compile (Ann _ rest) = compile rest
+    compile (Sym _ "" rest) = compile rest
     -- XXX: pragmas?
     compile (Var name) = return $ PVar name
     compile (Syn (sigil:"::()") exps) = do
@@ -371,8 +378,11 @@ compConditional exp = compError exp
 
 {-| Compiles various 'Exp's to 'PIL_Expr's. -}
 instance Compile Exp PIL_Expr where
-    compile (Ann (Pos _) rest) = compile rest -- fmap (PPos pos rest) $ compile rest
+    compile (Ann Pos{} rest) = compile rest -- fmap (PPos pos rest) $ compile rest
+    compile (Ann Prag{} rest) = compile rest
     compile (Ann (Cxt cxt) rest) = enter cxt $ compile rest
+    compile (Ann _ rest) = compile rest
+    compile (Sym _ "" rest) = compile rest
     -- XXX: pragmas?
     compile (Var name) = return . PExp $ PVar name
     compile exp@(Val (VCode _)) = compile $ Syn "sub" [exp]
@@ -403,6 +413,11 @@ compError = die $ "Compile error -- invalid "
 
 {-| Compiles a 'Val' to a 'PIL_Literal'. -}
 instance Compile Val PIL_Literal where
+    compile (VList vs) = return $ PVal (VList (filter isSimple vs))
+        where
+        isSimple (VRef _) = False
+        isSimple _        = True
+    compile (VRef _) = return $ PVal VUndef
     compile val = return $ PVal val
 
 -- utility functions
