@@ -64,8 +64,7 @@ compile (App (Var ('&':op)) Nothing [arg1, arg2]) = [| do
     |] where
     argC1 = compile arg1
     argC2 = compile arg2
-compile (Cxt _ arg) = compile arg
-compile (Pos _ arg) = compile arg
+compile (Ann _ arg) = compile arg
 compile (Val (VInt i)) = [| return (VInt i) |]
 compile (Val (VStr s)) = [| return (VStr s) |]
 compile (Val (VBool b)) = [| return (VBool b) |]
