@@ -12,15 +12,15 @@ class (Typeable a) => CodeClass a where
     code_type     :: a -> SubType
 
 instance CodeClass ICode where
-    code_iType c  = code_iType . unsafePerformSTM $ readTVar c
+    code_iType c  = code_iType . inlinePerformSTM $ readTVar c
     code_fetch    = liftSTM . readTVar
     code_store    = (liftSTM .) . writeTVar
     code_assuming c [] [] = code_fetch c
     code_assuming _ _ _   = error "assuming"
     code_apply    = error "apply"
-    code_assoc c  = code_assoc . unsafePerformSTM $ readTVar c
-    code_params c = code_params . unsafePerformSTM $ readTVar c
-    code_type c   = code_type . unsafePerformSTM $ readTVar c
+    code_assoc c  = code_assoc . inlinePerformSTM $ readTVar c
+    code_params c = code_params . inlinePerformSTM $ readTVar c
+    code_type c   = code_type . inlinePerformSTM $ readTVar c
 
 instance CodeClass VCode where
     -- XXX - subType should really just be a mkType itself
